from unittest import mock
from unittest.mock import mock_open, Mock

from pyerge import tools


def test_synced(str_sync):
    with mock.patch('pyerge.tools.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_sync.split('\n'))))
        assert tools.e_sync() == 'Tuesday 22:01' or 'Tuesday 23:01'


def test_no_synced(str_no_sync):
    with mock.patch('pyerge.tools.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_no_sync.split('\n'))))
        assert tools.e_sync() == 'Unknown'


def test_synced_ver1(str_sync):
    with mock.patch('pyerge.tools.open', mock_open(read_data=str_sync)) as m:
        result = tools.e_sync()

    m.assert_called_once_with(file='/var/log/emerge.log', encoding='utf-8')
    assert result == 'Tuesday 22:01' or 'Tuesday 23:01'


def test_e_dl_gt_zero(str_dl_gt_0):
    with mock.patch('pyerge.tools.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_dl_gt_0.split('\n'))))
        assert tools.e_dl() == '283,699 KiB'


def test_e_dl_eq_zero(str_dl_eq_0):
    with mock.patch('pyerge.tools.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_dl_eq_0.split('\n'))))
        assert tools.e_dl() == 'None'


def test_e_dl_unknown(str_dl_unknown):
    with mock.patch('pyerge.tools.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_dl_unknown.split('\n'))))
        assert tools.e_dl() == 'Calculating...'


def test_e_curr(str_curr):
    with mock.patch('pyerge.tools.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_curr.split('\n'))))
        assert tools.e_curr() == 'sys-kernel/linux-firmware-20191008'


def test_e_curr_empty(str_curr_empty):
    with mock.patch('pyerge.tools.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_curr_empty.split('\n'))))
        assert tools.e_curr() == ''


def test_e_eut(str_eut):
    with mock.patch('pyerge.tools.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_eut.split('\n'))))
        assert tools.e_eut() == '1h 9min'


def test_e_eut_unknown(str_eut_unknown):
    with mock.patch('pyerge.tools.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_eut_unknown.split('\n'))))
        assert tools.e_eut() == 'Unknown'


def test_e_eta_no_working_merge():
    with mock.patch('pyerge.tools.run_cmd') as run_cmd_mock:
        run_cmd_mock.return_value = b'!!! Error: no working merge found.\n' \
                                    b'(the -c option only works if there is an ongoing compilation, see manpage)\n', b''
        assert tools.e_eta() == 'Unknown'


def test_e_eta_emerge_working():
    with mock.patch('pyerge.tools.run_cmd') as run_cmd_mock:
        run_cmd_mock.return_value = b'\n Currently merging 1 out of 1\n\n' \
                                    b' * app-text/openjade-1.3.2-r9\n\n' \
                                    b'       current merge time: 9 seconds.\n' \
                                    b'       ETA: 1 minute and 21 seconds.\n', b''
        assert tools.e_eta() == '1 minute and 21 seconds'


def test_e_log(str_dl_gt_0):
    with mock.patch('pyerge.tools.open', mock_open(read_data=str_dl_gt_0)):
        result = tools.e_log()
    assert result == str_dl_gt_0


def test_e_prog_5_of_6(str_e_prog_5_6):
    with mock.patch('pyerge.tools.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_e_prog_5_6.split('\n'))))
        assert tools.e_prog() == 83.3333


def test_e_prog_1_of_1(str_e_prog_1_1):
    with mock.patch('pyerge.tools.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_e_prog_1_1.split('\n'))))
        assert tools.e_prog() == 100.0


def test_e_sta_compiling(str_e_sta_compiling):
    with mock.patch('pyerge.tools.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_e_sta_compiling.split('\n'))))
        assert tools.e_sta() == 'Compiling'


def test_e_sta_cleaning(str_e_sta_cleaning):
    with mock.patch('pyerge.tools.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_e_sta_cleaning.split('\n'))))
        assert tools.e_sta() == 'Cleaning'


def test_e_sta_autoclean(str_e_sta_autoclean):
    with mock.patch('pyerge.tools.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_e_sta_autoclean.split('\n'))))
        assert tools.e_sta() == 'Autoclean'


def test_e_sta_completed_emerge(str_e_sta_completed_emerge):
    with mock.patch('pyerge.tools.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_e_sta_completed_emerge.split('\n'))))
        assert tools.e_sta() == 'Completed'


def test_e_sta_finished(str_e_sta_finished):
    with mock.patch('pyerge.tools.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_e_sta_finished.split('\n'))))
        assert tools.e_sta() == 'Finished'


def test_e_sta_starting_rysnc(str_e_sta_starting_rsync):
    with mock.patch('pyerge.tools.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_e_sta_starting_rsync.split('\n'))))
        assert tools.e_sta() == "Syncing"


def test_e_sta_rysnc_completed(str_e_sta_sync_completed):
    with mock.patch('pyerge.tools.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_e_sta_sync_completed.split('\n'))))
        assert tools.e_sta() == "Synced"


def test_e_sta_unmerging(str_e_sta_unmerging):
    with mock.patch('pyerge.tools.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_e_sta_unmerging.split('\n'))))
        assert tools.e_sta() == 'Unmerging'


def test_e_sta_merging(str_e_sta_merging):
    with mock.patch('pyerge.tools.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_e_sta_merging.split('\n'))))
        assert tools.e_sta() == 'Merging'


def test_e_sta_unmerge(str_e_sta_unmerge):
    with mock.patch('pyerge.tools.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_e_sta_unmerge.split('\n'))))
        assert tools.e_sta() == 'Unmerge'


def test_e_sta_unknown(str_e_sta_unknown):
    with mock.patch('pyerge.tools.open') as open_mock:
        open_mock.return_value.__enter__ = open_mock
        open_mock.return_value.__iter__ = Mock(return_value=iter(list(str_e_sta_unknown.split('\n'))))
        assert tools.e_sta() == 'Unknown'


def test_e_upd_calculating(str_e_upd_calc):
    with mock.patch('pyerge.tools.open', mock_open(read_data=str_e_upd_calc)):
        result = tools.e_upd()
    assert result == 'Calculating...'


def test_e_upd_none(str_e_upd_none):
    with mock.patch('pyerge.tools.open', mock_open(read_data=str_e_upd_none)):
        result = tools.e_upd()
    assert result == 'None'


def test_e_upd_total(str_e_upd_total):
    with mock.patch('pyerge.tools.open', mock_open(read_data=str_e_upd_total)):
        result = tools.e_upd()
    assert result == '327 U, 10 N, 1 NS, 9 R, 2 Un'


def test_e_upd_conflict(str_e_upd_conflict):
    with mock.patch('pyerge.tools.open', mock_open(read_data=str_e_upd_conflict)):
        result = tools.e_upd()
    assert result == '327 U, 10 N, 2 NS, 9 R, 2 Un, 2 B'


def test_e_raid_match():
    with mock.patch('pyerge.tools.run_cmd') as run_cmd_mock:
        run_cmd_mock.return_value = b'Personalities : [linear] [raid0] [raid1] [raid10] [raid6] [raid5] [raid4] \n' \
                                    b'md16 : active raid5 sdc1[4] sdb1[1] sdd1[3]\n' \
                                    b'      1935409152 blocks super 1.2 level 5, 512k chunk, algorithm 2 [3/3] [_UU]\n' \
                                    b'      bitmap: 1/8 pages [4KB], 65536KB chunk\n\n' \
                                    b'md17 : active raid5 sdc2[4] sdb2[1] sdd2[3]\n' \
                                    b'      17835008 blocks super 1.2 level 5, 512k chunk, algorithm 2 [3/3] [UU_]\n' \
                                    b'      \nunused devices: <none>\n', b''
        assert tools.e_raid('md16') == '[_UU]'
        assert tools.e_raid('md17') == '[UU_]'


def test_e_raid_not_match():
    with mock.patch('pyerge.tools.run_cmd') as run_cmd_mock:
        run_cmd_mock.return_value = b'', b''
        assert tools.e_raid('md16') == 'Unknown'



from pytest import fixture


@fixture
def str_no_sync():
    return """1548194358:  === sync
1548194358: >>> Syncing repository 'gentoo' into '/usr/portage'...
1548194361: >>> Starting rsync with rsync://[2a01:90:200:10::1a]/gentoo-portage"""


@fixture
def str_sync():
    return """1548194361: >>> Starting rsync with rsync://[2a01:90:200:10::1a]/gentoo-portage
1548194466: === Sync completed for gentoo
1548194466:  *** terminating."""


@fixture
def str_curr():
    return """1571086865:  *** emerge --nospinner --oneshot sys-kernel/linux-firmware
1571086944:  >>> emerge (1 of 1) sys-kernel/linux-firmware-20191008 to /
1571086944:  === (1 of 1) Cleaning (sys-kernel/linux-firmware-20191008::/usr/portage/sys-kernel/linux-firmware/linux-firmware-20191008.ebuild)
1571086985:  === (1 of 1) Compiling/Merging (sys-kernel/linux-firmware-20191008::/usr/portage/sys-kernel/linux-firmware/linux-firmware-20191008.ebuild)
1571086996:  === (1 of 1) Merging (sys-kernel/linux-firmware-20191008::/usr/portage/sys-kernel/linux-firmware/linux-firmware-20191008.ebuild)
1571087012:  >>> AUTOCLEAN: sys-kernel/linux-firmware:0
1571087012:  === Unmerging... (sys-kernel/linux-firmware-20191004)"""


@fixture
def str_curr_empty():
    return """1571086865:  *** emerge --nospinner --oneshot sys-kernel/linux-firmware
1571086944:  >>> emerge (1 of 1) sys-kernel/linux-firmware-20191008 to /
1571086944:  === (1 of 1) Cleaning (sys-kernel/linux-firmware-20191008::/usr/portage/sys-kernel/linux-firmware/linux-firmware-20191008.ebuild)
1571086996:  === (1 of 1) Merging (sys-kernel/linux-firmware-20191008::/usr/portage/sys-kernel/linux-firmware/linux-firmware-20191008.ebuild)
1571087012:  >>> AUTOCLEAN: sys-kernel/linux-firmware:0
1571087012:  === Unmerging... (sys-kernel/linux-firmware-20191004)"""


@fixture
def str_dl_gt_0():
    return """[ebuild     U  ] xfce-extra/thunar-archive-plugin-0.4.0::gentoo [0.3.1-r3::gentoo] 342 KiB
[ebuild     U  ] xfce-extra/thunar-shares-plugin-0.3.0::gentoo [0.2.0_p20101105-r1::gentoo] 333 KiB
[ebuild     U  ] xfce-base/xfce4-meta-4.14-r1::gentoo [4.12-r1::gentoo] USE="svg upower%* -minimal" 0 KiB

Total: 44 packages (42 upgrades, 2 new), Size of downloads: 283,699 KiB
"""


@fixture
def str_dl_eq_0():
    return "Total: 0 packages, Size of downloads: 0 KiB"


@fixture
def str_dl_unknown():
    return ""


@fixture
def str_eut():
    return """[ebuild     U  ] xfce-base/xfdesktop-4.14.1::gentoo [4.12.4::gentoo] USE="libnotify thunar -debug" 1,498 KiB
[ebuild     U  ] xfce-extra/thunar-archive-plugin-0.4.0::gentoo [0.3.1-r3::gentoo] 342 KiB
[ebuild     U  ] xfce-extra/thunar-shares-plugin-0.3.0::gentoo [0.2.0_p20101105-r1::gentoo] 333 KiB
[ebuild     U  ] xfce-base/xfce4-meta-4.14-r1::gentoo [4.12-r1::gentoo] USE="svg upower%* -minimal" 0 KiB

!!! Error: couldn't get previous merge of libXpresent; skipping...

Estimated update time: 1 hour, 9 minutes.
"""


@fixture
def str_eut_unknown():
    return """[ebuild     U  ] xfce-base/xfdesktop-4.14.1::gentoo [4.12.4::gentoo] USE="libnotify thunar -debug" 1,498 KiB
[ebuild     U  ] xfce-extra/thunar-archive-plugin-0.4.0::gentoo [0.3.1-r3::gentoo] 342 KiB
[ebuild     U  ] xfce-extra/thunar-shares-plugin-0.3.0::gentoo [0.2.0_p20101105-r1::gentoo] 333 KiB
[ebuild     U  ] xfce-base/xfce4-meta-4.14-r1::gentoo [4.12-r1::gentoo] USE="svg upower%* -minimal" 0 KiB

!!! Error: couldn't get previous merge of libXpresent; skipping...
"""


@fixture
def orginal_html_xml():
    html = b'<?xml version="1.0" encoding="UTF-8"?>\n<rdf:RDF xmlns="http://purl.org/rss/1.0/"\n  ' \
           b'xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"\n  ' \
           b'xmlns:content="http://purl.org/rss/1.0/modules/content/"\n  ' \
           b'xmlns:dc="http://purl.org/dc/elements/1.1/"\n  ' \
           b'xmlns:image="http://purl.org/rss/1.0/modules/image/"\n  ' \
           b'xmlns:slash="http://purl.org/rss/1.0/modules/slash/"\n  ' \
           b'xmlns:sy="http://purl.org/rss/1.0/modules/syndication/"\n  ' \
           b'xmlns:taxo="http://purl.org/rss/1.0/modules/taxonomy/"\n  ' \
           b'xmlns:trackback="http://madskills.com/public/xml/rss/module/trackback/">\n' \
           b'<channel rdf:about="https://security.gentoo.org/glsa">\n    ' \
           b'<title>Gentoo Linux security advisories</title>\n    ' \
           b'<link>https://security.gentoo.org/glsa</link>\n    ' \
           b'<description>This feed contains new Gentoo Linux security advisories. ' \
           b'Contact security@gentoo.org with questions.</description>\n' \
           b'<items>\n      <rdf:Seq>\n' \
           b'<rdf:li resource="https://security.gentoo.org/glsa/201904-25"/>\n' \
           b'<rdf:li resource="https://security.gentoo.org/glsa/201904-24"/>\n' \
           b'<rdf:li resource="https://security.gentoo.org/glsa/201904-23"/>\n' \
           b'</items>\n    <dc:date>2019-04-24T00:00:00+00:00</dc:date>\n  </channel>\n' \
           b'<item rdf:about="https://security.gentoo.org/glsa/201904-25">\n    ' \
           b'<title>GLSA 201904-25: QEMU: Multiple vulnerabilities</title>\n    ' \
           b'<link>https://security.gentoo.org/glsa/201904-25</link>\n    ' \
           b'<dc:date>2019-04-24T00:00:00+00:00</dc:date>\n  </item>\n' \
           b'<item rdf:about="https://security.gentoo.org/glsa/201904-24">\n    ' \
           b'<title>GLSA 201904-24: Ming: Multiple vulnerabilities</title>\n    ' \
           b'<link>https://security.gentoo.org/glsa/201904-24</link>\n    ' \
           b'<dc:date>2019-04-24T00:00:00+00:00</dc:date>\n  </item>\n' \
           b'<item rdf:about="https://security.gentoo.org/glsa/201904-23">\n    ' \
           b'<title>GLSA 201904-23: GLib: Multiple vulnerabilities</title>\n    ' \
           b'<link>https://security.gentoo.org/glsa/201904-23</link>\n    ' \
           b'<dc:date>2019-04-22T00:00:00+00:00</dc:date>\n  </item>\n' \
           b'</rdf:RDF>'
    return html


@fixture()
def html_xlm_as_str():
    return """
<?xml version="1.0" encoding="UTF-8"?>
<rdf:RDF xmlns="http://purl.org/rss/1.0/"
  xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
  xmlns:content="http://purl.org/rss/1.0/modules/content/"
  xmlns:dc="http://purl.org/dc/elements/1.1/"
  xmlns:image="http://purl.org/rss/1.0/modules/image/"
  xmlns:slash="http://purl.org/rss/1.0/modules/slash/"
  xmlns:sy="http://purl.org/rss/1.0/modules/syndication/"
  xmlns:taxo="http://purl.org/rss/1.0/modules/taxonomy/"
  xmlns:trackback="http://madskills.com/public/xml/rss/module/trackback/">
  <channel rdf:about="https://security.gentoo.org/glsa">
    <title>Gentoo Linux security advisories</title>
    <link>https://security.gentoo.org/glsa</link>
    <description>This feed contains new Gentoo Linux security advisories. Contact security@gentoo.org with questions.</description>
    <items>
      <rdf:Seq>
        <rdf:li resource="https://security.gentoo.org/glsa/201904-23"/>
        <rdf:li resource="https://security.gentoo.org/glsa/201904-22"/>
      </rdf:Seq>
    </items>
    <dc:date>2019-04-24T00:00:00+00:00</dc:date>
  </channel>
  <item rdf:about="https://security.gentoo.org/glsa/201904-23">
    <title>GLSA 201904-23: GLib: Multiple vulnerabilities</title>
    <link>https://security.gentoo.org/glsa/201904-23</link>
    <dc:date>2019-04-22T00:00:00+00:00</dc:date>
  </item>
  <item rdf:about="https://security.gentoo.org/glsa/201904-22">
    <title>GLSA 201904-22: OpenDKIM: Root privilege escalation</title>
    <link>https://security.gentoo.org/glsa/201904-22</link>
    <dc:date>2019-04-22T00:00:00+00:00</dc:date>
  </item>
</rdf:RDF>
"""


@fixture()
def opt_emerge_nonlocal_with_1g():
    from argparse import Namespace
    return Namespace(action='emerge', local=False, size='1G', verbose=2)


@fixture
def str_e_upd_calc():
    return """"""


@fixture
def str_e_upd_none():
    return "'Total: 0 packages, Size of downloads: 0 KiB'"


@fixture
def str_e_upd_total():
    return """[blocks b      ] kde-apps/kcalcore:5 ("kde-apps/kcalcore:5" is blocking kde-frameworks/kcalendarcore-5.64.0)
[ebuild     U  ] kde-apps/libkgapi-19.08.3:5::gentoo [19.04.3:5::gentoo] USE="nls -debug -test" 229 KiB
[ebuild     U  ] kde-misc/kio-gdrive-1.2.7:5::gentoo [1.2.6:5::gentoo] USE="handbook kaccounts -debug -test" 46 KiB

Total: 347 packages (327 upgrades, 10 new, 1 in new slot, 9 reinstalls, 2 uninstalls), Size of downloads: 642,505 KiB
"""


@fixture
def str_e_upd_conflict():
    return """[blocks b      ] kde-apps/kcalcore:5 ("kde-apps/kcalcore:5" is blocking kde-frameworks/kcalendarcore-5.64.0)
[ebuild     U  ] kde-apps/libkgapi-19.08.3:5::gentoo [19.04.3:5::gentoo] USE="nls -debug -test" 229 KiB
[ebuild     U  ] kde-misc/kio-gdrive-1.2.7:5::gentoo [1.2.6:5::gentoo] USE="handbook kaccounts -debug -test" 46 KiB

Total: 347 packages (327 upgrades, 10 new, 2 in new slots, 9 reinstalls, 2 uninstalls), Size of downloads: 642,505 KiB
Conflict: 2 blocks
"""


@fixture
def str_e_prog_5_6():
    return """1637618618:  >>> emerge (5 of 6) www-client/vivaldi-4.3.2439.71-r1 to /
1637618618:  === (5 of 6) Cleaning (www-client/vivaldi-4.3.2439.71-r1::/usr/portage/www-client/vivaldi/vivaldi-4.3.2439.71-r1.ebuild)
1637618619:  === (5 of 6) Compiling/Merging (www-client/vivaldi-4.3.2439.71-r1::/usr/portage/www-client/vivaldi/vivaldi-4.3.2439.71-r1.ebuild)
1637618637:  === (5 of 6) Merging (www-client/vivaldi-4.3.2439.71-r1::/usr/portage/www-client/vivaldi/vivaldi-4.3.2439.71-r1.ebuild)
1637618646:  >>> AUTOCLEAN: www-client/vivaldi:0
1637618646:  === Unmerging... (www-client/vivaldi-4.3.2439.63)
1637618651:  >>> unmerge success: www-client/vivaldi-4.3.2439.63
1637618659:  === (5 of 6) Post-Build Cleaning (www-client/vivaldi-4.3.2439.71-r1::/usr/portage/www-client/vivaldi/vivaldi-4.3.2439.71-r1.ebuild)
1637618659:  ::: completed emerge (5 of 6) www-client/vivaldi-4.3.2439.71-r1 to /"""


@fixture
def str_e_prog_1_1():
    return """1637631921:  *** emerge --regex-search-auto=y dev-python/pyerge
1637631930:  >>> emerge (1 of 1) dev-python/pyerge-0.4.0 to /
1637631930:  === (1 of 1) Cleaning (dev-python/pyerge-0.4.0::/usr/local/portage/dev-python/pyerge/pyerge-0.4.0.ebuild)
1637631930:  === (1 of 1) Compiling/Merging (dev-python/pyerge-0.4.0::/usr/local/portage/dev-python/pyerge/pyerge-0.4.0.ebuild)
1637631937:  === (1 of 1) Merging (dev-python/pyerge-0.4.0::/usr/local/portage/dev-python/pyerge/pyerge-0.4.0.ebuild)
1637631941:  >>> AUTOCLEAN: dev-python/pyerge:0"""


@fixture
def str_e_sta_compiling():
    return """sr/portage/dev-perl/Mozilla-CA/Mozilla-CA-20999999-r1.ebuild)
1637618600:  === (4 of 6) Compiling/Merging (dev-perl/Mozilla-CA-20999999-r1::/usr/portage/dev-perl/Mozilla-CA/Mozilla-CA-20999999-r1.ebuild)
1637618606:  === (4 of """


@fixture
def str_e_sta_cleaning():
    return """Compat-0.130.0-r1 to /
1637618599:  === (4 of 6) Cleaning (dev-perl/Mozilla-CA-20999999-r1::/usr/portage/dev-perl/Mozilla-CA/Mozilla-CA-20999999-r1.ebuild)
1637618600:  === (4 of 6)"""


@fixture
def str_e_sta_autoclean():
    return """hesis-6.25.0::/usr/portage/dev-python/hypothesis/hypothesis-6.25.0.ebuild)
1637618672:  >>> AUTOCLEAN: dev-python/hypothesis:0
1637618672:  === """


@fixture
def str_e_sta_completed_emerge():
    return """v-perl/Mozilla-CA/Mozilla-CA-20999999-r1.ebuild)
1637618618:  ::: completed emerge (4 of 6) dev-perl/Mozilla-CA-20999999-r1 to /
1637618618:  === (5 of 6"""


@fixture
def str_e_sta_finished():
    return """othesis-6.25.0 to /
1637618682:  *** Finished. Cleaning up...
1637618686:  *** terminating.
1637631776:  *** em"""


@fixture
def str_e_sta_starting_rsync():
    return """pository 'gentoo' into '/usr/portage'...
1548195587: >>> Starting rsync with rsync://[2a00:1828:a00d:ffff::6]/gentoo-portage
1548195642: === Sync com"""


@fixture
def str_e_sta_sync_completed():
    return """1604197444: >>> Syncing repository 'gentoo' into '/usr/portage'...
1604197446: >>> Starting rsync with rsync://[2001:470:1f14:104f::2]/gentoo-portage
1604197788: === Sync completed for gentoo
1604197788: >>> Syncing repository 'tor"""


@fixture
def str_e_sta_unmerging():
    return """perl/Mozilla-CA:0
1637618610:  === Unmerging... (dev-perl/Mozilla-CA-20999999)
1637618618:  === (4 of 6"""


@fixture
def str_e_sta_merging():
    return """zilla-CA-20999999-r1::/usr/portage/dev-perl/Mozilla-CA/Mozilla-CA-20999999-r1.ebuild)
1637618606:  === (4 of 6) Merging (dev-perl/Mozilla-CA-20999999-r1::/usr/portage/dev-perl/Mozilla-CA/Mozilla-CA-20999999-r1.ebuild)
1637618610:  >>>"""


@fixture
def str_e_sta_unmerge():
    return """erging... (dev-perl/Tie-IxHash-1.230.0)
1604167130:  >>> unmerge success: dev-perl/Tie-IxHash-1.230.0
1604167130:  *** exiting successfully.
1604167131:  *** terminating."""


@fixture
def str_e_sta_unknown():
    return """portage/eix:0
1548195548:  === (3 of 3) Updating world file (app-portage/eix-0.33.5)
1548195548:  === (3 of 3) Post-Build Cle"""



from argparse import Namespace
from logging import warning, info, debug
from os import system, environ
from re import search
from shlex import split
from subprocess import Popen, PIPE  # nosec
from typing import Union, Tuple

from pyerge import PORTAGE_TMPDIR


def run_cmd(cmd: str, use_system=False) -> Tuple[bytes, bytes]:
    """
    Run any system command.

    If use_system is set cmd is run via os.system and function
    return RC from comand as bytes and b''.
    If use_system is not set (default) cmd is run via subprocess.Popen and
    function return cmd stdout and stderr as bytes.

    :param cmd: command string
    :param use_system: os.system use insted of subprocess
    :return: tuple of bytes with output and error
    """
    if use_system:
        ret_code = system(cmd)  # nosec
        out, err = str(ret_code).encode('utf-8'), b''
    else:
        out, err = Popen(split(cmd), stdout=PIPE, stderr=PIPE).communicate()  # nosec
    return out, err


def mounttmpfs(size: str) -> None:
    """
    Mount directory with size as tmp file system in RAM.

    :param size: with unit K, M, G
    """
    info(f'Mounting {size} of memory to {PORTAGE_TMPDIR}')
    debug(f'sudo mount -t tmpfs -o size={size},nr_inodes=1M tmpfs {PORTAGE_TMPDIR}')
    run_cmd(f'sudo mount -t tmpfs -o size={size},nr_inodes=1M tmpfs {PORTAGE_TMPDIR}')


def unmounttmpfs(opts: Namespace) -> None:
    """
    Unmount directory from RAM.

    :param opts: cli arguments
    """
    if not opts.action == 'check' and (not opts.local or not opts.pretend_world):
        info(f'Unmounting {opts.size} of memory from {PORTAGE_TMPDIR}')
        debug(f'sudo umount -f {PORTAGE_TMPDIR}')
        run_cmd(f'sudo umount -f {PORTAGE_TMPDIR}')


def remounttmpfs(size: str) -> None:
    """
    Re-mount directory with size as tmp file system in RAM.

    :param size: with unit K, M, G
    """
    info(f'Remounting {size} of memory to {PORTAGE_TMPDIR}')
    debug(f'sudo umount -f {PORTAGE_TMPDIR}')
    run_cmd(f'sudo umount -f {PORTAGE_TMPDIR}')
    debug(f'sudo mount -t tmpfs -o size={size},nr_inodes=1M tmpfs {PORTAGE_TMPDIR}')
    run_cmd(f'sudo mount -t tmpfs -o size={size},nr_inodes=1M tmpfs {PORTAGE_TMPDIR}')


def is_internet_connected() -> bool:
    """
    Check if there is connection to internet.

    :return: True is connected, False otherwise
    """
    cmd, _ = run_cmd('ping -W1 -c1 89.16.167.134')
    match = search(b'[1].*, [1].*, [0]%.*,', cmd)
    if match is not None:
        info('There is internet connecton or not needed')
        return True
    warning('No internet connection!')
    return False


def size_of_mounted_tmpfs() -> int:
    """
    Return size of mounted directory.

    :return: size in bytes as intiger
    """
    df_cmd, _ = run_cmd('df')
    match = search(rf'(tmpfs\s*)(\d+)(\s*.*{PORTAGE_TMPDIR})', df_cmd.decode('utf-8'))
    if match is not None:
        return int(match.group(2))
    return 0


def is_tmpfs_mounted() -> bool:
    """
    Check if portage temp dir is mounted.

    :return: True is mounted, False otherwise
    """
    mount_cmd, _ = run_cmd('mount')
    match = search(rf'(tmpfs on\s+)({PORTAGE_TMPDIR})(\s+type tmpfs)', mount_cmd.decode('utf-8'))
    return bool(match is not None and match.group(2) == PORTAGE_TMPDIR)


def convert2blocks(size: str) -> int:
    """
    Convert size with unit into system blocks (used in i.e. df/mounts commands).

    :param size: with units K, M, G
    :return: size in kB
    """
    try:
        return int(float(size))
    except ValueError as err:
        debug(f'Size: {size} Exception: {err}')
    regex = search(r'(?i)(\d+)([KMG])', size)
    map_dict = {'K': 1, 'M': 1024, 'G': 1024 * 1024}
    return int(regex.group(1)) * map_dict[regex.group(2).upper()]  # type: ignore


def delete_content(fname: Union[str, bytes, int]) -> None:
    """
    Clean-up file content.

    :param fname: path to file as string
    """
    with open(file=fname, mode='w', encoding='utf-8'):
        pass


def set_portage_tmpdir() -> str:
    """Set system variable."""
    if not environ.get('PORTAGE_TMPDIR', ''):
        environ['PORTAGE_TMPDIR'] = PORTAGE_TMPDIR
    return environ['PORTAGE_TMPDIR']


def handling_mounting(opts: Namespace) -> None:
    """
    Handling mounting temporary file fistem with requestes size.

    :param opts: cli arguments
    """
    if not opts.action == 'check' and (not opts.local or not opts.pretend_world):
        if not is_tmpfs_mounted():
            mounttmpfs(opts.size)
        elif size_of_mounted_tmpfs() != convert2blocks(opts.size):
            remounttmpfs(opts.size)
        else:
            info('tmpfs is already mounted with requested size!')



from argparse import ArgumentParser
from datetime import datetime
from re import search, sub, match

from pyerge import EMERGE_LOGFILE, TMERGE_LOGFILE, TMPLOGFILE
from pyerge.utils import run_cmd


def e_sync() -> str:
    """Fetch date of last sync form logs."""
    with open(file=EMERGE_LOGFILE, encoding='utf-8') as log_file:
        for line in list(log_file)[::-1]:
            reqex = search(r'(\d+)(:\s===\sSync completed)', line)
            if reqex is not None:
                sync_time = reqex.group(1)
                break
        else:
            return 'Unknown'
    sync_date = datetime.fromtimestamp(int(sync_time)).strftime('%A %H:%M')
    print(sync_date)
    return sync_date


def e_dl() -> str:
    """Fetch size of archives to be downloaded for next system update."""
    with open(file=TMERGE_LOGFILE, mode='r', encoding='utf-8') as log_file:
        for line in list(log_file)[::-1]:
            reqex = search(r'(Size of downloads:.)([0-9,]*\s[KMG]iB)', line)
            if reqex is not None:
                size = reqex.group(2)
                break
        else:
            size = None  # type: ignore

    if size == '0 KiB':
        size = 'None'
    elif not size:
        size = 'Calculating...'
    print(size)
    return size


def e_curr() -> str:
    """Get name of the current or last compiled package."""
    with open(file=EMERGE_LOGFILE, encoding='utf-8') as log_file:
        for line in list(log_file)[::-1]:
            reqex = search(r'Compiling.*\((.*)::', line)
            if reqex is not None:
                pack = reqex.group(1)
                break
        else:
            pack = ''
    print(pack)
    return pack


def e_eut() -> str:
    """Get estimated update time."""
    with open(file=TMPLOGFILE, encoding='utf-8') as log_file:
        for line in list(log_file)[::-1]:
            reqex = search(r'Estimated update time:\s+(.*)\.', line)
            if reqex is not None:
                eut = reqex.group(1).replace(',', '')
                eut = sub(' minutes| minute', 'min', eut)
                eut = sub(' hours| hour', 'h', eut)
                break
        else:
            eut = 'Unknown'
    print(eut)
    return eut


def e_eta() -> str:
    """Get estimated time of compilation of current package."""
    output, _ = run_cmd('genlop -cn')
    out = output.decode('utf-8')
    eta = ''
    if search(r'Error.*no working merge found', out):
        eta = 'Unknown'

    reqex = search(r'ETA:\s(.*)\.', out)
    if reqex is not None:
        eta = reqex.group(1)
    print(eta)
    return eta


def e_sta() -> str:
    """Get current stage of emerging package."""
    result = 'Unknown'
    map_dict = {'Compiling': 'Compiling', 'Cleaning': 'Cleaning', 'AUTOCLEAN': 'Autoclean', 'completed emerge': 'Completed', 'Finished': 'Finished',
                'Sync completed': 'Synced', 'Starting rsync': 'Syncing', 'Unmerging': 'Unmerging', 'Merging': 'Merging', 'unmerge': 'Unmerge'}
    with open(file=EMERGE_LOGFILE, encoding='utf-8') as log_file:
        emerge_log = ''.join(list(log_file)[::-1][:16])
    regex = search(r'(Compiling|Cleaning|AUTOCLEAN|completed\semerge|Finished|Starting\srsync|Sync\scompleted|Unmerging|Merging|unmerge)', emerge_log)
    if regex is not None:
        result = map_dict[regex.group(1)]
    print(result)
    return result


def e_prog() -> float:
    """Get current progress of emerging packages."""
    result = 100.0
    with open(file=EMERGE_LOGFILE, encoding='utf-8') as log_file:
        emerge_log = ''.join(list(log_file)[::-1][:51])
    regex = search(r'\(([0-9]*)\sof\s([0-9]*)\)', emerge_log)
    if regex is not None:
        result = round(100.0 * int(regex.group(1)) / int(regex.group(2)), 4)
        print(result)
    return result


def e_log() -> str:
    """Check next update content."""
    with open(file=TMERGE_LOGFILE, encoding='utf-8') as log_file:
        content = log_file.read()
    return content


def e_upd() -> str:
    """Check types and number of packages to update."""
    result = 'Calculating...'
    map_dict = {'upgrades': 'U', 'upgrade': 'U', 'new': 'N', 'in new slot': 'NS', 'in new slots': 'NS', 'reinstalls': 'R',
                'reinstall': 'R', 'uninstalls': 'Un', 'uninstall': 'Un', 'downgrades': 'D', 'downgrade': 'D', 'blocks': 'B', 'block': 'B'}
    with open(file=TMERGE_LOGFILE, encoding='utf-8') as log_file:
        content = log_file.read()

    if search(r'Total: 0 packages, Size of downloads: 0 KiB', content):
        result = "None"
    regex_total = search(r'Total:\s\d*\spackages?\s\((.*)\),.*', content)
    regex_conflict = search(r'Conflict:\s(\d*\sblocks?)', content)
    total_list = regex_total.group(1).split(',') if regex_total else []
    conflict_list = regex_conflict.group(1).split(',') if regex_conflict else []
    if total_list:
        list_str = [element.strip() for element in [*total_list, *conflict_list]]
        upd_dict = {match(r'\d*\s([A-Za-z ]*)', element).group(1): match(r'(\d*)\s\w*', element).group(1) for element in list_str}  # type: ignore
        result = ', '.join([f'{v} {map_dict[k]}' for k, v in upd_dict.items() if k in map_dict])
    print(result)
    return result


def e_raid(raid_id: str) -> str:
    """
    Check of Raid array.

    :param raid_id: name i.e. md126 or md127
    :return: status of RAID
    """
    raid = 'Unknown'
    out, _ = run_cmd('cat /proc/mdstat')
    out = out.decode('utf-8')  # type: ignore
    reqex = search(rf'{raid_id}.*\n.*(\[[U_]*\])', out)  # type: ignore
    if reqex is not None:
        raid = reqex.group(1)  # type: ignore
    return raid


def run_e_raid():
    """Run e_raid from cli."""
    parser = ArgumentParser()
    parser.add_argument('-n', '--name', action='store', help='Provides name of MD RAID Array')
    args = parser.parse_args()
    print(e_raid(args.name))



from unittest.mock import patch

from pytest import raises, mark


def test_cli_system_exit():
    with raises(SystemExit) as sys_mock:
        from pyerge import cli
        cli.run_parser()
    assert sys_mock.value.code > 0


def test_run_parser_with_correct_action():
    from argparse import Namespace, ArgumentParser
    with patch.object(ArgumentParser, 'parse_known_args') as argument_parser_mock, patch('pyerge.cli.main_exec') as main_exec_mock:
        opts = Namespace(action='emerge', world=True, verbose=1, quiet=True)
        emerge_opts = ['-NDu', '@world']
        argument_parser_mock.return_value = (opts, emerge_opts)
        from pyerge import cli
        cli.run_parser()
        main_exec_mock.assert_called_once_with(opts, emerge_opts)


def test_main_exec_portage_is_running():
    from pyerge import utils, tmerge
    from argparse import Namespace
    with patch.object(utils, 'is_internet_connected') as is_internet_connected_mock, patch.object(tmerge, 'is_portage_running') as is_portage_running_mock:
        is_internet_connected_mock.return_value = True
        is_portage_running_mock.return_value = True
        from pyerge import cli
        opts = Namespace(world=False, pretend_world=False, pretend=False, quiet=False, verbose=0)
        emerge_opts = ['']
        cli.main_exec(opts, emerge_opts)
        is_internet_connected_mock.assert_called_once_with()
        is_portage_running_mock.assert_called_once()


@mark.parametrize('ns_args, emerge_opts, result',
                  [({'world': True, 'pretend_world': False, 'pretend': False, 'quiet': False, 'verbose': 2}, [''], ['-NDu', '@world']),
                   ({'world': True, 'pretend_world': False, 'pretend': False, 'quiet': True, 'verbose': 2}, [''], ['-NDuq', '@world']),
                   ({'world': False, 'pretend_world': True, 'pretend': False, 'quiet': False, 'verbose': 2}, [''], ['-pvNDu', '@world']),
                   ({'world': False, 'pretend_world': True, 'pretend': False, 'quiet': True, 'verbose': 2}, [''], ['-pvNDuq', '@world']),
                   ({'world': False, 'pretend_world': False, 'pretend': True, 'quiet': False, 'verbose': 2}, ['-av', 'app-misc/mc'], ['-avp', 'app-misc/mc']),
                   ({'world': False, 'pretend_world': False, 'pretend': False, 'quiet': True, 'verbose': 2}, ['-av', 'app-misc/mc'], ['-avq', 'app-misc/mc'])])
def test_emerge_opts_world(ns_args, emerge_opts, result):
    from pyerge import utils, tmerge
    from argparse import Namespace
    with patch.object(utils, 'is_internet_connected') as is_internet_connected_mock, \
            patch.object(utils, 'set_portage_tmpdir') as set_portage_tmpdir_mock, \
            patch.object(utils, 'handling_mounting') as handling_mounting_mock, \
            patch.object(utils, 'unmounttmpfs') as unmounttmpfs_mock, \
            patch.object(tmerge, 'is_portage_running') as is_portage_running_mock, \
            patch.object(tmerge, 'run_emerge') as run_emerge_mock, \
            patch.object(tmerge, 'run_check') as run_check_mock:
        is_internet_connected_mock.return_value = False
        is_portage_running_mock.return_value = False
        from pyerge import cli
        opts = Namespace(**ns_args)
        cli.main_exec(opts, emerge_opts)
        is_internet_connected_mock.assert_called_once_with()
        set_portage_tmpdir_mock.assert_called_once()
        handling_mounting_mock.assert_called_once_with(opts)
        unmounttmpfs_mock.assert_called_once_with(opts)
        is_portage_running_mock.assert_called_once()
        run_emerge_mock.assert_called_once_with(result, opts)
        run_check_mock.assert_called_once_with(opts)



__version__ = '0.5.0'

PORTAGE_TMPDIR = '/var/tmp/portage'  # nosec
TMPLOGFILE = '/var/log/portage/tmerge/tmp.emerge.log'
TMERGE_LOGFILE = '/var/log/portage/tmerge/emerge.log'
EMERGE_LOGFILE = '/var/log/emerge.log'
DEVNULL = '/dev/null 2>&1'



from unittest import mock

from pytest import mark

date1 = '201904-23'
date2 = '201904-22'
date3 = '201904-13'
date4 = '201904-14'
vulnerabilities = f'{date1}: GLib: Multiple vulnerabilities'
escalation = f'{date2}: OpenDKIM: Root privilege escalation'


def test_glsa_list():
    from pyerge.glsa import glsa_list
    with mock.patch('pyerge.glsa._rss') as rss_mock:
        rss_mock.return_value = [vulnerabilities, escalation]
        assert glsa_list(elements=2) == f'{vulnerabilities}\n{escalation}'


def test_glsa_test_system_not_affected():
    from pyerge.glsa import glsa_test
    with mock.patch('pyerge.glsa.utils') as utils_mock:
        with mock.patch('pyerge.glsa._rss') as rss_mock:
            rss_mock.return_value = [date1, date2]
            utils_mock.run_cmd.return_value = b'', b'This system is not affected by any of the listed GLSAs\n'
            assert glsa_test(elements=2) == 'System is not affected by any of listed GLSAs'


def test_glsa_test_system_affected():
    from pyerge.glsa import glsa_test
    with mock.patch('pyerge.glsa.utils') as utils_mock:
        with mock.patch('pyerge.glsa._rss') as rss_mock:
            rss_mock.return_value = [date1, date2, date3]
            utils_mock.run_cmd.return_value = b'201904-13\n201904-14\n', b'This system is affected by the following GLSAs:\n'
            assert glsa_test(elements=2) == f'{date3},{date4}'


def test_rss(orginal_html_xml):
    from pyerge.glsa import _rss
    with mock.patch('pyerge.glsa.request') as urlopen_mock:
        with mock.patch('pyerge.glsa._collect_all_maching_entries') as collect_all_mock:
            urlopen_mock.return_value.read.return_value = orginal_html_xml
            collect_all_mock.return_value = ['201904-25', '201904-24', '201904-23']
            assert _rss(r'GLSA\s(\d{6}-\d{2}):\s.*', 2) == ['201904-25', '201904-24']


@mark.parametrize('regex, result', [(r'GLSA\s(\d{6}-\d{2}):\s.*', [date1, date2]),
                                    (r'GLSA\s(\d{6}-\d{2}:\s.*)', [vulnerabilities, escalation])])
def test_collect_all_maching_entries(regex, result, html_xlm_as_str):
    from pyerge.glsa import _collect_all_maching_entries
    assert _collect_all_maching_entries(html_xlm_as_str, regex) == result
